==> org-lsst-ccs-subsystem-demo-main-6.1.0-sources.jar: org/lsst/ccs/subsystem/demo/main/device/IonPumps.java <==
package org.lsst.ccs.subsystem.demo.main.device;

import java.time.Duration;
import java.util.Random;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.lsst.ccs.commons.annotations.LookupField;
import org.lsst.ccs.commons.annotations.LookupPath;
import org.lsst.ccs.drivers.commons.DriverException;
import org.lsst.ccs.drivers.demo.DemoDriver;
import org.lsst.ccs.framework.AgentPeriodicTask;
import org.lsst.ccs.monitor.Device;
import org.lsst.ccs.services.AgentPeriodicTaskService;
import org.lsst.ccs.services.DataProviderDictionaryService;


/**
 * Monitoring Device. It publishes a metadata called 'devicemetadata' at its
 * level, that will therefore be associated to all its channels.
 *
 * @author turri
 */
public class IonPumps extends Device {
    
    @LookupField(strategy=LookupField.Strategy.TREE)
    private DataProviderDictionaryService dataProviderDictionaryService;
    
    private DemoDriver driver;
    private final String id;
    private final Random r = new Random();
    private int meta;
    
    @LookupPath
    private String path;

    public IonPumps(String id) {
        this.id = id;
    }

    public String getId() {
        return id;
    }
    
    @Override
    public void build() {
        s.getAgentService(AgentPeriodicTaskService.class).scheduleAgentPeriodicTask(
        new AgentPeriodicTask(path + "-update-metadata",
        () -> { 
            meta+=2;
            dataProviderDictionaryService.publishMetadataForObject("devicemetadata", String.valueOf(meta), this);
        }).withPeriod(Duration.ofSeconds(-1))
        );
    }

    @Override
    protected void initialize() {
        if (driver == null) {
            try {
                driver = new DemoDriver();
            } catch (DriverException ex) {
                Logger.getLogger(IonPumps.class.getName()).log(Level.SEVERE, null, ex);
                throw new RuntimeException(ex);
            }
        }
        setOnline(true);
    }

    @Override
    protected void close() {
        if (driver != null) {
            try {
                driver.close();
                driver = null;
            } catch (DriverException ex) {
                Logger.getLogger(IonPumps.class.getName()).log(Level.SEVERE, null, ex);
                throw new RuntimeException(ex);
            }
        }
    }

    @Override
    protected double readChannel(int chan, int type) {
        if (chan < 10) {
            return r.nextGaussian()*.5 + chan;
        } else {
            return r.nextGaussian()*10 + (chan%10)*1000;
        }
    }
    
}

==> org-lsst-ccs-subsystem-demo-main-6.1.1-sources.jar: org/lsst/ccs/subsystem/demo/main/device/IonPumps.java <==
package org.lsst.ccs.subsystem.demo.main.device;

import java.time.Duration;
import java.util.Random;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.lsst.ccs.commons.annotations.LookupField;
import org.lsst.ccs.commons.annotations.LookupPath;
import org.lsst.ccs.drivers.commons.DriverException;
import org.lsst.ccs.drivers.demo.DemoDriver;
import org.lsst.ccs.framework.AgentPeriodicTask;
import org.lsst.ccs.monitor.Channel;
import org.lsst.ccs.monitor.Device;
import org.lsst.ccs.services.AgentPeriodicTaskService;
import org.lsst.ccs.services.DataProviderDictionaryService;


/**
 * Monitoring Device. It publishes a metadata called 'devicemetadata' at its
 * level, that will therefore be associated to all its channels.
 *
 * @author turri
 */
public class IonPumps extends Device {
    
    @LookupField(strategy=LookupField.Strategy.TREE)
    private DataProviderDictionaryService dataProviderDictionaryService;
    
    private DemoDriver driver;
    private final String id;
    private final Random r = new Random();
    private int meta;
    
    @LookupPath
    private String path;

    public IonPumps(String id) {
        this.id = id;
    }

    public String getId() {
        return id;
    }
    
    @Override
    public void build() {
        s.getAgentService(AgentPeriodicTaskService.class).scheduleAgentPeriodicTask(
        new AgentPeriodicTask(path + "-update-metadata",
        () -> { 
            meta+=2;
            dataProviderDictionaryService.publishMetadataForObject("devicemetadata", String.valueOf(meta), this);
        }).withPeriod(Duration.ofSeconds(-1))
        );
    }

    @Override
    protected void initialize() {
        if (driver == null) {
            try {
                driver = new DemoDriver();
            } catch (DriverException ex) {
                Logger.getLogger(IonPumps.class.getName()).log(Level.SEVERE, null, ex);
                throw new RuntimeException(ex);
            }
        }
        setOnline(true);
    }

    @Override
    protected void close() {
        if (driver != null) {
            try {
                driver.close();
                driver = null;
            } catch (DriverException ex) {
                Logger.getLogger(IonPumps.class.getName()).log(Level.SEVERE, null, ex);
                throw new RuntimeException(ex);
            }
        }
    }

    @Override
    protected double readChannel(Channel ch) {
        int chan = ch.getHwChan();
        if (chan < 10) {
            return r.nextGaussian()*.5 + chan;
        } else {
            return r.nextGaussian()*10 + (chan%10)*1000;
        }
    }
    
}
